/* cdpd_snr.ps.c */                                                       
/* Custom Signal-to-Noise-Ratio (SNR) model for radio link Transceiver Pipeline */

/****************************************/
/*		Michael Jiang           */
/*		  10/08/1999            */
/****************************************/

#include <opnet.h>
#include <math.h>


void
cdpd_snr (pkptr)
	Packet*		pkptr;
	{
	double		bkg_noise, accum_noise, rcvd_power;
	double		total_noise;

	/** Compute the signal-to-noise ratio for the given packet. **/
	FIN (cdpd_snr (pkptr))

	/* Get the packet's received power level. */
	rcvd_power = op_td_get_dbl (pkptr, OPC_TDA_RA_RCVD_POWER);

	/* Get the packet's accumulated noise levels calculated by the */
	/* interference and background noise stages. */
	accum_noise = op_td_get_dbl (pkptr, OPC_TDA_RA_NOISE_ACCUM);
	bkg_noise = op_td_get_dbl (pkptr, OPC_TDA_RA_BKGNOISE);
	
	/* Convert the accum noise factors to dBm. */
	if (accum_noise > 0.0)
		accum_noise = 10.0 * log10 (1000.0 * accum_noise);

	else
		accum_noise = 0.0;

	/* Assume thermal noise of -129 dBm and received front end noise of -9dBm = -120 dBm. */
	total_noise = bkg_noise + accum_noise;

	/* Assign the SNR in dBm. */
	op_td_set_dbl (pkptr, OPC_TDA_RA_SNR, 
		(rcvd_power - (total_noise)));

	/*op_td_set_dbl (pkptr, OPC_TDA_RA_SNR, rcvd_power/3);*/
	/* Set field indicating the time at which SNR was calculated. */
	op_td_set_dbl (pkptr, OPC_TDA_RA_SNR_CALC_TIME, op_sim_time ());

	FOUT
	}

